# -*- coding: utf-8 -*-
"""
Created on Thu May 31 23:12:02 2012

@author: Federico Lora
"""


import numpy as np
#whether or not to use the 3d visualization
UseVisual = True
if UseVisual:
    #import custom visualization library
    import atomwrite

import ProjectMC2 as ProjectMC

def InitPositions(N, L,r0,delr):
    """Returns an array of initial positions of each atom,
placed on a cubic lattice for convenience.
Input:
    N: number of atoms
    L: box length
Output:
    Pos: (N,3) array of positions
"""
    #make the position array
    Pos = np.zeros((N,3), float)
    #compute integer grid # of locations for cubic lattice
    NLat = int(N**(1./2.) + 1.)
    LatSpac = L / NLat
    #make an array of lattice sites
    r = LatSpac * np.arange(NLat, dtype=float) - 0.5*L
    #loop through x, y, z positions in lattice until done
    #for every atom in the system
    i = 0
    for x in r:
        for y in r:
                Pos[i] = np.array([x,y,r0+delr*np.abs(np.random.rand())], float)
                #add a random offset to help initial minimization
                Offset = 0.1 * LatSpac * (np.random.rand(3) - 0.5)
                Pos[i] = Pos[i] + Offset
                i += 1
                #if done placing atoms, return
                if i >= N:
                    return Pos
    return Pos

    
def MCstep(N,L,T,Pos,delr, rc, eps, g):
    accept = 0
    dU = 0
    # choose the atom
    MoveAtom = np.random.randint(len(Pos)) 
    # copy atom position before the move        
    OldPos = Pos[MoveAtom,:].copy()
    # choose random move
    Move = (delr+delr)*np.random.random(len(Pos))-delr
    k = 0
    # calculate the single atom energy before move
    U1 = ProjectMC.singleenergy(Pos,MoveAtom,L,rc,eps, g,3,N)
    while k < 3:
        # move atom (update position)            
        Pos[MoveAtom,k] = Pos[MoveAtom,k] + Move[k]
        k += 1 
        
    # re-calculate the single atom energy after move
    U2 = ProjectMC.singleenergy(Pos,MoveAtom,L,rc,eps, g, 3,N)
    if U2 < U1:
        # if lower in energy, accept move and update PE
        dU = U2-U1
        accept += 1
    else:
        #roll the dice!            
        Pacc = np.exp(-(U2-U1)/T)
        r = np.random.random()
        if Pacc > r:
        # if accept: update PE
            dU = U2-U1
            accept += 1.           
        else:
            Pos[MoveAtom,:] = OldPos
    return Pos, dU, accept            

def MCpartC1(sweeps, grav,P):
    global N,L,rc,eps,g
    N = 50
    T =3.0
    eps = -50
    iterations = sweeps*N
    rescalefrequency = 250
    for g in [grav]:
        G = -g*eps
        rc = 1.01
        for rho in [P]:
            i = 0            
            L = (1*N/rho)**(1./3.) 
            delr = 0.01
            if rho == 0.001:
                delr = 0.05
            if rho == .01:
                delr = 0.03
            if rho == .05:
                delr = 0.025
            if rho == .3:
                delr = 0.01
            filename = 'MonteCarloMOVIE'+str(g)+'_'+str(rho)+'.txt'
            l = [L,L,4*L]
            Pdb = atomwrite.pdbfile(filename, l)
            F = file(filename,"w")            
            PE = 1e17
            Pos = InitPositions(N,L,2,2.2)
            PE = ProjectMC.calcenergy(Pos,L,rc,eps, G, 3,N)
            print PE
            # Monte Carlo: #
            accept = 0     
            #atomvis.Init(Pos,2*L)
            while i < iterations:
                if PE > 600000:
                    delr = 0.1
                if PE < 100000:
                    delr = 0.015
                Pos, dU, A = MCstep(N,L,T,Pos,delr, rc, eps, G)
                PE = PE + dU
                accept = accept + A
                if i % rescalefrequency == 0:          
                    PE = ProjectMC.calcenergy(Pos,L,rc,eps,G,3,N)
                    Pdb.write(Pos)          
                    #atomvis.Update(Pos)
                    k = 0
                    count = 0
                    while k < N:
                        if Pos[k,2]<1.1:
                            count +=1
                        k +=1
                    Fraction = count*(3.14156*0.5**2)/(L**2)    
                    fract = str(Fraction)                    
                    I = str(i)
                    potE = str(PE)
                    F.write(I)
                    F.write(' ')
                    F.write(potE)
                    F.write(' ')
                    F.write(fract)
                    F.write(' ')                     
                    F.write('\n')
                    print I, PE, Fraction, np.min(Pos[:,2]),np.max(Pos[:,2]), L
                i +=1
            acceptance = accept/(iterations)
            AA = str(acceptance)
            F.write(AA)            
            F.close()
            Pdb.close()
            print g, rho, acceptance,PE
    return Pos, L    

def runMC(sweeps):
    global N,L,rc,eps,g
    for grav in [3]:
        for P in [.15]:
            Pos,L = MCpartC1(sweeps,grav,P)
            Dist = ProjectMC.distances(Pos,L)
            Counts, bins = np.histogram(Dist,10)
            filename = 'MonteCarloMOVIE'+str(grav)+'_'+str(P)+'Histogram'+'.txt'
            F = file(filename,"w")
            j = 0
            while j < 10:
                c = str(Counts[j])
                b = str(bins[j])
                F.write(b)
                F.write(' ')
                F.write(c)
                F.write('\n')
                j += 1
            F.close()
    return()

                